#include "flacDecoder.hpp"

#include <math.h>
#include <string.h>

#include <iostream>

#define PLAYBACK_SAMPLERATE 48000

#define RAW_DATA_MILLISECONDS 5000

bool FlacDecoder::getMoreData()
{
    int copyAmount = overflowSamplesLength;
    if (copyAmount > rawDataAllocatedLength - rawDataLength)
    {
        copyAmount = rawDataAllocatedLength - rawDataLength;
    }

    if (copyAmount != 0)
    {
        int CopyAmountReduced = std::min(rawDataAllocatedLength - rawDataWritePosition, copyAmount);

        memcpy(rawData + rawDataWritePosition, overflowSamples + overflowSamplesReadPosition, CopyAmountReduced * sizeof(float));
        if (CopyAmountReduced != copyAmount)
        {
            memcpy(rawData, overflowSamples + overflowSamplesReadPosition + CopyAmountReduced, (copyAmount - CopyAmountReduced) * sizeof(float));
        }

        unreadSamplesLeft -= copyAmount;

        overflowSamplesLength -= copyAmount;
        rawDataLength += copyAmount;
        rawDataWritePosition += copyAmount;
        if (rawDataWritePosition >= rawDataAllocatedLength) // here it is actually needed to have >= because the whole overflow sample length may be added to it
        {
            rawDataWritePosition -= rawDataAllocatedLength;
        }
    }

    if (rawDataAllocatedLength != rawDataLength)
    {
        overflowSamplesReadPosition = 0;

        if (process_single() == false)
        {
            FLAC__StreamDecoderState state = get_state();
            std::cerr << "error processing flac frames: " << FLAC__StreamDecoderStateString[state] << std::endl;
            return false;
        }
    }
    else
    {
        overflowSamplesReadPosition += copyAmount;
    }
    return true;
}

FLAC__StreamDecoderWriteStatus FlacDecoder::write_callback(const ::FLAC__Frame *frame, const FLAC__int32 *const buffer[])
{
    for (uint32_t i = 0; i < frame->header.blocksize; i++)
    {
        if (rawDataLength == rawDataAllocatedLength)
        {
            uint32_t dataLeft = (frame->header.blocksize - i) * 2;
            for (; i < frame->header.blocksize; i++)
            {
                if (channels == 1)
                {
                    overflowSamples[overflowSamplesLength] = (float)buffer[0][i] / (1 << bps);
                    overflowSamples[overflowSamplesLength + 1] = (float)buffer[0][i] / (1 << bps);
                }
                else
                {
                    overflowSamples[overflowSamplesLength] = (float)buffer[0][i] / (1 << bps);
                    overflowSamples[overflowSamplesLength + 1] = (float)buffer[1][i] / (1 << bps);
                }
                overflowSamplesLength += 2;
            }
            break;
        }
        if (channels == 1)
        {
            rawData[rawDataWritePosition] = (float)buffer[0][i] / (1 << bps);
            rawData[rawDataWritePosition + 1] = (float)buffer[0][i] / (1 << bps);
        }
        else // always use max first 2 channels
        {
            rawData[rawDataWritePosition] = (float)buffer[0][i] / (1 << bps);
            rawData[rawDataWritePosition + 1] = (float)buffer[1][i] / (1 << bps);
        }

        rawDataLength += 2;
        rawDataWritePosition += 2;
        unreadSamplesLeft -= 2;
        if (rawDataWritePosition == rawDataAllocatedLength)
        {
            rawDataWritePosition = 0;
        }
    }

    return FLAC__STREAM_DECODER_WRITE_STATUS_CONTINUE;
}

// we don't need to check for the type of the callback since by default this callback is only called with type streaminfo
void FlacDecoder::metadata_callback(const ::FLAC__StreamMetadata *metadata)
{
    bps = metadata->data.stream_info.bits_per_sample;
    sampleRate = metadata->data.stream_info.sample_rate;
    channels = metadata->data.stream_info.channels;
    totalSamples = metadata->data.stream_info.total_samples;
    unreadSamplesLeft = totalSamples * 2;

    std::cout << "bps: " << bps << "\nsamplerate: " << sampleRate << "\nchannels: " << channels << "\ntotalSamples: " << totalSamples << "\nmax blocksize: " << metadata->data.stream_info.max_blocksize << std::endl;

    overflowSamplesAllocatedLength = metadata->data.stream_info.max_blocksize * 2; // 2 for channel count
    overflowSamples = new float[overflowSamplesAllocatedLength];

    rawDataAllocatedLength = 2 * RAW_DATA_MILLISECONDS * sampleRate / 1000;
    rawData = new float[rawDataAllocatedLength];
}

void FlacDecoder::error_callback(::FLAC__StreamDecoderErrorStatus status)
{
    // TODO: actually handle errors
    std::cerr << "Got error callback" << FLAC__StreamDecoderErrorStatusString[status] << std::endl;
}

FlacDecoder::FlacDecoder()
{
}

FlacDecoder::~FlacDecoder()
{
    delete overflowSamples;
    delete rawData;
}