#pragma once

#include <samplerate.h>
#include <stddef.h>
#include <stdint.h>

#include <atomic>
#include <chrono>
#include <condition_variable>
#include <functional>
#include <mutex>
#include <thread>

#include "audioProvider.hpp"
#include "flacDecoder.hpp"

#define RESAMPLER_TYPE SRC_SINC_MEDIUM_QUALITY

class FlacProvider : public AudioProvider
{
public:
    FlacProvider(std::string filepath, std::chrono::milliseconds _startTime, std::function<void()> _onEndedCallback);
    ~FlacProvider();
    void getData(int16_t *audioData, size_t sampleCount, size_t sampleIndex, int64_t samplePosition);
    void waitForInitialPreprosessing();

private:
    bool loadFile(std::string filepath);
    void preprocessData();
    bool initResampler();

    std::chrono::milliseconds startTime;

    bool unconvertedAudioLeft = true;

    FlacDecoder flacDecoder;

    SRC_STATE *resamplerState;
    SRC_DATA resamplerData;

    double resamplingRatio;

    float *preloadedDataBuffer;
    // it is assumed that all of these aare divisible by 2 (2 channel audio)               // 1 frame is 2 samples (2 channel audio)
    int preloadedDataAllocatedLength = 0;
    std::atomic<int> preloadedDataLength = 0; // has to be atomic as the playback thread and the preprocessing thread modifies this at the same time
    int preloadedDataReadPosition = 0;
    int preloadedDataWritePosition = 0;

    bool shouldRemove = false;

    std::function<void()> onEndedCallback;

    std::thread preprocessingThread;
    std::mutex preprocessingMutex;
    std::condition_variable preprocessingConditionVariable;
    bool preprocessingFinished = false;
    std::atomic<bool> shouldExitPreprocessingThread = false;
};