#include "fileSystem.hpp"

#include <openssl/evp.h>

#include <chrono>
#include <filesystem>
#include <fstream>
#include <iostream>
#include <string>
#include <vector>

void removeFile(std::string name)
{
    std::remove(name.c_str());
}

void moveFile(std::string oldName, std::string newName)
{
    std::rename(oldName.c_str(), newName.c_str());
}

void appendToFile(std::string name, unsigned char *data, uint16_t length)
{
    std::ofstream file("music/" + name, std::ios::binary | std::ios::app);
    file.write(reinterpret_cast<char *>(data), length);
    file.close();
}

std::string getFileHash(std::string filename)
{
    std::ifstream file("music/" + filename, std::ios::binary);
    if (!file.is_open())
    {
        std::cerr << "could not open file: " << filename << std::endl;
        return "";
    }

    EVP_MD_CTX *mdctx;
    unsigned char *md5_digest;
    unsigned int md5_digest_len = EVP_MD_size(EVP_md5());

    mdctx = EVP_MD_CTX_new();
    EVP_DigestInit_ex(mdctx, EVP_md5(), NULL);

    unsigned char buffer[1024];
    while (!file.eof())
    {
        file.read(reinterpret_cast<char *>(buffer), 1024);
        EVP_DigestUpdate(mdctx, buffer, file.gcount());
    }

    md5_digest = (unsigned char *)OPENSSL_malloc(md5_digest_len); // TODO: Idk what to do if memory allocation fails
    EVP_DigestFinal_ex(mdctx, md5_digest, &md5_digest_len);
    EVP_MD_CTX_free(mdctx);

    std::stringstream ss;
    for (int i = 0; i < md5_digest_len; i++)
    {
        ss << std::hex << std::setw(2) << std::setfill('0') << (int)md5_digest[i];
    }
    OPENSSL_free(md5_digest);
    return ss.str();
}

std::vector<std::string> getFiles()
{
    std::vector<std::string> files;
    for (auto &entry : std::filesystem::directory_iterator("music/"))
    {
        files.push_back(entry.path().filename());
    }
    return files;
}

void createDirectories()
{
    std::filesystem::create_directory("music/");
}