#pragma once

#include <cerrno>

// Check if we're using glibc >= 2.32 where strerrorname_np is available
#if defined(__GLIBC__) && (__GLIBC__ >= 2) && (__GLIBC_MINOR__ >= 32)
// No need for a polyfill, strerrorname_np is available
#else
// Define the polyfill if strerrorname_np is not available
const char* strerrorname_np(int errnum)
{
    switch (errnum)
    {
        case EACCES:
            return "EACCES";
        case EADDRINUSE:
            return "EADDRINUSE";
        case EADDRNOTAVAIL:
            return "EADDRNOTAVAIL";
        case EAFNOSUPPORT:
            return "EAFNOSUPPORT";
        case EAGAIN:
            return "EAGAIN";
        case EALREADY:
            return "EALREADY";
        case EBADF:
            return "EBADF";
        case ECONNABORTED:
            return "ECONNABORTED";
        case ECONNREFUSED:
            return "ECONNREFUSED";
        case ECONNRESET:
            return "ECONNRESET";
        case EDESTADDRREQ:
            return "EDESTADDRREQ";
        case EHOSTUNREACH:
            return "EHOSTUNREACH";
        case EINPROGRESS:
            return "EINPROGRESS";
        case EINVAL:
            return "EINVAL";
        case EISCONN:
            return "EISCONN";
        case ENETDOWN:
            return "ENETDOWN";
        case ENETRESET:
            return "ENETRESET";
        case ENETUNREACH:
            return "ENETUNREACH";
        case ENOBUFS:
            return "ENOBUFS";
        case ENOTCONN:
            return "ENOTCONN";
        case ENOTSOCK:
            return "ENOTSOCK";
        case EOPNOTSUPP:
            return "EOPNOTSUPP";
        case EOVERFLOW:
            return "EOVERFLOW";
        case EPIPE:
            return "EPIPE";
        case EPROTO:
            return "EPROTO";
        case EPROTONOSUPPORT:
            return "EPROTONOSUPPORT";
        case EPROTOTYPE:
            return "EPROTOTYPE";
        case ETIMEDOUT:
            return "ETIMEDOUT";
        default:
            return "Unknown error";
    }
}
#endif