#pragma once

#include <condition_variable>
#include <iostream>
#include <mutex>

class TCPClient
{
  private:
    enum SocketState
    {
        DISCONNECTED,
        DISCONNECTING,
        CONNECTED,
        CONNECTING,
    };

    std::atomic<int> socketfd;

    std::mutex socketOperationMutex;

    std::mutex socketReceiveMutex;

    std::atomic<bool> disconnectedForcefully;

    std::atomic<SocketState> socketState;

    std::condition_variable finishedCV;

  public:
    TCPClient() : socketfd(-1), socketState(SocketState::DISCONNECTED) {}
    ~TCPClient();
    TCPClient(const TCPClient&) = delete;
    TCPClient& operator=(const TCPClient&) = delete;
    TCPClient(TCPClient&&) = delete;
    TCPClient& operator=(TCPClient&&) = delete;

    // Is not multithreaded safe. Only one thread should call this function anzyways
    void connect(std::string host, std::string port);

    // Is multithreaded safe
    void disconnect();

    // Is multithreaded safe
    void send(const char* data, size_t size, bool corc = false);

    // Is not multithreaded safe. Only one thread should call this function anyways
    size_t receive(char* data, size_t size);

    class ConnectException : public std::runtime_error
    {
      public:
        ConnectException(std::string message) : std::runtime_error(message) {}
    };

    class SendException : public std::runtime_error
    {
      public:
        SendException(std::string message) : std::runtime_error(message) {}
    };

    class ReceiveException : public std::runtime_error
    {
      public:
        ReceiveException(std::string message) : std::runtime_error(message) {}
    };

    class NotConnectedException : public std::runtime_error
    {
      public:
        NotConnectedException(std::string message) : std::runtime_error(message) {}
    };
};
