#include "playbackSystem.hpp"

#include <iostream>
#include <map>
#include <mutex>

#include "../audioProviders/flacProvider.hpp"
#include "../audioSystem.hpp"

#define AUDIO_PREFIX_PATH std::string("music/")

static std::map<uint64_t, AudioProvider *> audioProviders;
static std::recursive_mutex audioProvidersMutex;

void addNewMusic(uint64_t id, std::string filename, std::chrono::milliseconds startTime)
{
    std::lock_guard<std::recursive_mutex> lock(audioProvidersMutex);

    if (audioProviders.count(id) > 0)
    {
        std::cout << "Recreating audio provider for id " << id << std::endl;
        // TODO: Implement reassigning of existing start time.
        // for now just recreate the flac provider

        removeMusic(id);
    }
    std::cout << "Creating new provider with id " << id << " and filename " << filename << std::endl;

    FlacProvider *flacProvider = new FlacProvider(AUDIO_PREFIX_PATH + filename, startTime, [id, filename]() {
        std::cout << "Playback time with id " << id << " and filename " << filename << " ended" << std::endl;
        removeMusic(id);
    });

    flacProvider->waitForInitialPreprosessing();

    addAudioProvider(flacProvider);
    audioProviders[id] = flacProvider;

    std::cout << "Added provider " << id << " to audio providers" << std::endl;
}

void removeMusic(uint64_t id)
{
    std::lock_guard<std::recursive_mutex> lock(audioProvidersMutex);

    if (audioProviders.count(id) == 0)
    {
        // Can happen if some external source tries to remove a music at the same time as for example the flac
        // provider onEndedCallback is trying to remove itself.
        // Also normal for a server to request stopping of a already stopped song.
        return;
    }

    std::cout << "remove provider " << id << " from audio providers" << std::endl;
    AudioProvider *audioProvider = audioProviders[id];
    audioProviders.erase(id);

    removeAudioProvider(audioProvider);
    delete audioProvider;
}
