#include <sys/wait.h>

#include <cstring>
#include <iostream>

#include "audioProviders/flacProvider.hpp"
#include "audioSystem.hpp"
#include "crashHandler.hpp"
#include "fileSystem.hpp"
#include "networking/networking.hpp"
#include "playbackSystem/playbackSystem.hpp"

static std::string defaultPort = "13173";

void printVersion()
{
    std::cerr << "Poppi " << VERSION << std::endl;
}

ExecutionParameters parseExecutionParameters(int argc, char **argv)
{
    ExecutionParameters args;

    args.userExecutedBinary = argv[0];
    if (getenv("ARGV0") != nullptr)
    {
        args.userExecutedBinary = getenv("ARGV0");
        // TODO: use these too for other things
        // getenv("APPIMAGE") // full path to the appimage the user executed
        // getenv("APPDIR") // full path to the directory where the appimage is mounted

        // Also the appimage-type2-runtime (and also the old type 1) has a bug where
        // the environment variables are not set correctly when the appimage "main" (AppRun)
        // is a script file (when the checkrt plugin is used).
    }

    args.serverPort = defaultPort;

    if (argc == 1)
    {
        std::cout << "Enter the server address: ";
        std::cin >> args.serverAddress;

        std::cout << "Using default port: " << defaultPort << std::endl;
    }
    else if (argc == 2)
    {
        args.serverAddress = argv[1];

        std::cout << "Using default port: " << defaultPort << std::endl;
    }
    else if (argc == 3)
    {
        args.serverAddress = argv[1];
        args.serverPort = argv[2];

        if (args.serverPort.find_first_not_of("0123456789") != std::string::npos)
        {
            std::cerr << "Port must be a number" << std::endl;
            exit(1);
        }
        // verify that port is in the valid range
        if (std::stoi(args.serverPort) < 1 || std::stoi(args.serverPort) > 65535)
        {
            std::cerr << "Port must be in the range 1-65535" << std::endl;
            exit(1);
        }
    }
    else
    {
        std::cerr << "Usage: " << argv[0] << " <address> [port]" << std::endl;
        exit(1);
    }

    return args;
}

int poppiMain(ExecutionParameters params)
{
    createDirectories();

    networking::start(params.serverAddress, params.serverPort);

    if (startAudioSystem() == false)
    {
        return 0;
    }

    // TODO: make the shutdown process make sense
    shutdownAudioSystem();

    return 0;
}

int main(int argc, char **argv)
{
    printVersion();
    ExecutionParameters params = parseExecutionParameters(argc, argv);

    runWithCrashHandler(params, poppiMain, printVersion);
}